 /**
 * <p>Qualitaets- und UnterstuetzungsAgentur - Landesinstitut fuer Schule, Materialien zum schulinternen Lehrplan Informatik SII</p>
 *
 * @version 1.1; April 2014
 */

import java.io.*;

/**
 * Ein Wettkampfergebnis verwaltet Wettkampfkarten. <br>
 * Es kann der beste Sportler einer oder aller Disziplinen ermittelt werden. <br>
 * Die Sportler koennen nach ihren Leistungen sortiert werden.
 */
public class Wettkampfergebnis {

	
	public static final int AUSWAHL = 1, EINFUEGEN = 2, BUBBLESORT = 3, QUICKSORT = 4, MERGESORT = 5;
   
	private List<Wettkampfkarte> alleWettkaempfe;

	private int anzahlWettkampfkarten;
	
	/**
	 * Eine leeres Wettkampfergebnis wurde erzeugt.
	 */
	public Wettkampfergebnis() {
		alleWettkaempfe = new List<>();
		anzahlWettkampfkarten = 0;
	}

   public void reset() {
		this.alleWettkaempfe.toFirst();
	}

	public void next() {
		this.alleWettkaempfe.next();
	}

	public boolean hasAccess() {
		return this.alleWettkaempfe.hasAccess();
	}

	public boolean isEmpty() {
		return this.alleWettkaempfe.isEmpty();
	}

	/**
	 * Diese Anfrage liefert die aktuelle Wettkampfkarte der verwalteten Liste.
	 */
	public Wettkampfkarte aktuelleWettkampfkarte() {
		return aktuelleWettkampfkarte (this.alleWettkaempfe);
	}
	
	private Wettkampfkarte aktuelleWettkampfkarte (List<Wettkampfkarte> karten) {
		return karten.getContent();
	}

	/**
	 * Die angegebene Wettkampfkarte wurde zum Ergebnis hinzugefgt.
	 */
	public void ergaenzeKarte(Wettkampfkarte pWettkampfkarte) {
		alleWettkaempfe.append(pWettkampfkarte);
		anzahlWettkampfkarten++;
	}

	/**
	 * Die Ergebnisse wurde nach der angegebenen Disziplin mit dem angegebenen
	 * Sortierverfahren sortiert.
	 */
	public void sortiereNach(int pDisziplin, int pVerfahren) {
		switch (pVerfahren) {
		case AUSWAHL:
			this.auswahlSort(pDisziplin);
			break;
		case EINFUEGEN:
			this.einfuegeSort(pDisziplin);
			break;
		case BUBBLESORT:
			this.bubblesort(pDisziplin);
			break;
         case QUICKSORT:  
            this.quicksort(pDisziplin); 
            break;
         case MERGESORT:  
            this.mergesort(pDisziplin); 
            break;
        }     
	}

	private void auswahlSort(int pDisziplin) {
		List<Wettkampfkarte> hilfsliste = new List<>();
        while (!this.alleWettkaempfe.isEmpty())
        {
            // Beste Karte heraussuchen
            Wettkampfkarte bester = this.besteKarte(pDisziplin);
            this.alleWettkaempfe.toFirst();
            while (this.aktuelleWettkampfkarte() != bester)
            {
            	this.alleWettkaempfe.next();
            }

            hilfsliste.append(bester);
            this.alleWettkaempfe.remove();
        }
        this.alleWettkaempfe = hilfsliste; 

	}

	private void einfuegeSort(int pDisziplin) {
        List<Wettkampfkarte> hilfsliste = new List<>();
        while (!this.alleWettkaempfe.isEmpty())
        {
        	this.alleWettkaempfe.toFirst();
            Wettkampfkarte lKarte = this.aktuelleWettkampfkarte();

            hilfsliste.toFirst();
            while (hilfsliste.hasAccess() && lKarte.gibPunkte(pDisziplin) < this.aktuelleWettkampfkarte(hilfsliste).gibPunkte(pDisziplin))
            {
            	hilfsliste.next();
            }
            if (hilfsliste.hasAccess())
            {
            	hilfsliste.insert(lKarte);
            }
            else
            {
            	hilfsliste.append(lKarte);
            }

            this.alleWettkaempfe.remove();
        }
        this.alleWettkaempfe = hilfsliste; 

	}

	private void bubblesort(int pDisziplin) {
        boolean getauscht;
        Wettkampfkarte blase, zweite;
        do
        {
            getauscht = false;
            this.alleWettkaempfe.toFirst();
            blase = this.aktuelleWettkampfkarte();
            this.alleWettkaempfe.remove();
            while (this.alleWettkaempfe.hasAccess())
            {
                zweite = this.aktuelleWettkampfkarte();
                if (blase.gibPunkte(pDisziplin) >= zweite.gibPunkte(pDisziplin))
                {
                	this.alleWettkaempfe.insert(blase);
                    blase = zweite;
                    this.alleWettkaempfe.remove();
                } 
                else
                {
                    getauscht = true;
                    this.alleWettkaempfe.next();
                }
            }
            this.alleWettkaempfe.append(blase);
        }
        while (getauscht);

	}

	
	/**
	 * Wenn das Ergebnis mindestens eine Wettkampfkarte enthaelt, wird die beste
	 * Karte bezueglich der angegebenen Disziplin geliefert. <br>
	 * Falls es mehrere Karten mit der gleichen Punktzahl in der angegebenen
	 * Disziplin gibt, ist es die zuerst gefundene.
	 */
	public Wettkampfkarte besteKarte(int pDisziplin) {
		this.alleWettkaempfe.toFirst();
		Wettkampfkarte bester = this.aktuelleWettkampfkarte();
		while (this.alleWettkaempfe.hasAccess()) {
			if (bester.gibPunkte(pDisziplin) < this.aktuelleWettkampfkarte()
					.gibPunkte(pDisziplin)) {
				bester = this.aktuelleWettkampfkarte();
			}
			this.alleWettkaempfe.next();
		}
		return bester;
	}


	private int length(List<Wettkampfkarte> l)
	{
	    l.toFirst();
	    int count = 0;
	    while (l.hasAccess()){
	        count++;
	        l.next();
	       }
	    return count;
	}
	
    private void quicksort(int pDisziplin)
    {
        this.quicksortRekursiv(alleWettkaempfe, pDisziplin);
    }

    private void quicksortRekursiv(List<Wettkampfkarte> pListe, int pDisziplin)
    {
        if (this.length(pListe) > 1)
        {
            pListe.toFirst();
            Wettkampfkarte trennelement = this.aktuelleWettkampfkarte(pListe);
            pListe.remove();
            List<Wettkampfkarte> groessere = new List<>();
            List<Wettkampfkarte> kleinere = new List<>();
            while (!pListe.isEmpty())
            {
                if (this.aktuelleWettkampfkarte(pListe).gibPunkte(pDisziplin) < trennelement.gibPunkte(pDisziplin))
                {
                    kleinere.append(this.aktuelleWettkampfkarte(pListe));
                }
                else
                {
                    groessere.append(this.aktuelleWettkampfkarte(pListe));
                }
                pListe.remove();
            }
            this.quicksortRekursiv(groessere, pDisziplin);
            this.quicksortRekursiv(kleinere, pDisziplin);
            pListe.concat(groessere);
            pListe.append(trennelement);
            pListe.concat(kleinere);
        }
    }

  

    private void mergesort(int pDisziplin)
    {
        this.mergesortRekursiv(alleWettkaempfe, pDisziplin);
    }

    private void mergesortRekursiv(List<Wettkampfkarte> pListe, int pDisziplin)
    {
        if (this.length(pListe) > 1)
        {
            int mitte = this.length(pListe) / 2;
            List<Wettkampfkarte> ersteHaelfte = new List<>();
            List<Wettkampfkarte> zweiteHaelfte = new List<>();   
            int nr = 1;
            pListe.toFirst();
            while (!pListe.isEmpty())
            {
                if (nr <= mitte)
                    ersteHaelfte.append(pListe.getContent());
                else
                    zweiteHaelfte.append(pListe.getContent());
                nr++;
                pListe.remove();
            }
            this.mergesortRekursiv(ersteHaelfte, pDisziplin);
            this.mergesortRekursiv(zweiteHaelfte, pDisziplin);
            this.mische(ersteHaelfte, zweiteHaelfte, pDisziplin);
            pListe.concat(ersteHaelfte);
        }
    }

    /**
     * vorher: Beide Listen sind absteigend sortiert.<br>
     * nachher: Die erste Liste enthlt die Elemente beider Listen und ist absteigend sortiert.
     */
    private void mische(List<Wettkampfkarte> pErsteListe, List<Wettkampfkarte> pZweiteListe, int pDisziplin)
    {
        pErsteListe.toFirst();
        pZweiteListe.toFirst();
        while (pZweiteListe.hasAccess())
        {
            if (pErsteListe.hasAccess())
            {
                if (this.aktuelleWettkampfkarte(pErsteListe).gibPunkte(pDisziplin) < this.aktuelleWettkampfkarte(pZweiteListe).gibPunkte(pDisziplin))
                {
                    pErsteListe.insert(this.aktuelleWettkampfkarte(pZweiteListe));
                    pZweiteListe.next();
                }
                else
                {
                    pErsteListe.next();
                }
            }
            else
            {
                pErsteListe.append(this.aktuelleWettkampfkarte(pZweiteListe));
                pZweiteListe.next();
            }
        }

    }
}